"""
Editor
"""

import pickle
import pygame, Numeric
from pygame.locals import *

import tile_engine

import psyco
#psyco.log()
psyco.profile()

class editor:
    def __init__(self):
        self.init_display()
        self.init_map()

        self.message_font = pygame.font.Font('data/font.ttf', 32)

        self.frame_count = 0
        self.cursor_colors = [(191, 191, 191), (63, 63, 63)]
        self.cursor_pos = None
        self.cur_tile = 0
        self.view_pos = (0, 0)
        self.full_draw_flag = 1
    
    def init_display(self):
        pygame.init()

        self.screen = pygame.display.set_mode((640, 480), HWSURFACE | DOUBLEBUF, 32)
        self.screen.blit(pygame.image.load("data/status_bar.png"), (0, 400))
        self.view = self.screen.subsurface(Rect(0, 0, 640, 400))
        
    def init_map(self):
        f = file("data/test.map", "r")
        self.t = tile_engine.tile_engine(pickle.load(f), "data/tiles")
        f.close()

    def build_new_map(self, w, h):
        r = []
        for i in range(w):
            r.append(0)
            
        self.map = []
        for i in range(h):
            self.t.map.append(list(r))
            
        self.t.map_tile_width = w
        self.t.map_tile_height = h

    def new_map(self):
        d = Rect(self.screen.get_width()/2-150, self.screen.get_height()/2-100, 300, 200)
        self.screen.fill((0, 0, 127), d)

        self.screen.blit(self.message_font.render("Width:", 1, (0, 127, 191), (0, 0, 127)), (200, 170))
        self.screen.blit(self.message_font.render("Height:", 1, (0, 127, 191), (0, 0, 127)), (200, 218))
        width_rect = Rect(330, 170, 72, 48)
        height_rect = Rect(330, 218, 72, 48)
        self.screen.fill((0, 0, 0), width_rect)
        self.screen.fill((0, 0, 0), height_rect)

        self.screen.blit(self.message_font.render("Ok", 1, (0, 127, 191), (0, 0, 127)), (220, 280))
        self.screen.blit(self.message_font.render("Cancel", 1, (0, 127, 191), (0, 0, 127)), (300, 280))
        ok_rect = Rect(210, 280, 63, 42)
        cancel_rect = Rect(290, 280, 127, 42)
        pygame.draw.rect(self.screen, (0, 0, 255), ok_rect, 2)
        pygame.draw.rect(self.screen, (0, 0, 255), cancel_rect, 2)

        ws = `self.t.map_tile_width`
        hs = `self.t.map_tile_height`
        cf = 0

        in_dialog = 1
        while in_dialog:
            e = pygame.event.poll()
            if e.type == QUIT or (e.type == KEYDOWN and e.key == K_ESCAPE):
                in_dialog = 0
            elif e.type == MOUSEBUTTONUP:
                if ok_rect.collidepoint(pygame.mouse.get_pos()):
                    self.build_new_map(int(ws), int(hs))
                    in_dialog = 0
                elif cancel_rect.collidepoint(pygame.mouse.get_pos()):
                    in_dialog = 0
            elif e.type == KEYDOWN:
                if e.key in range(K_0, K_9+1):
                    if cf == 0 and len(ws) < 3:
                        ws = ws + `e.key - K_0`
                    elif cf == 1 and len(hs) < 3:
                        hs = hs + `e.key - K_0`
                elif e.key == K_TAB:
                    cf = (cf + 1) % 2
                elif e.key == K_BACKSPACE:
                    if cf == 0:
                        ws = ws[:-1]
                    elif cf == 1:
                        hs = hs[:-1]
                elif e.key == K_RETURN:
                    self.build_new_map(int(ws), int(hs))
                    in_dialog = 0

            self.screen.fill(0, width_rect)
            self.screen.fill(0, height_rect)

            if len(ws) > 0:
                r = self.message_font.render(ws, 1, (0, 127, 191), (0, 0, 0))
                offset = r.get_width()
                self.screen.blit(r, (336, 170))
            else:
                offset = 0
            if cf == 0:
                self.screen.fill((0, 0, 255), Rect(336 + offset, 172, 2, 40))

            if len(hs) > 0:
                r = self.message_font.render(hs, 1, (0, 127, 191), (0, 0, 0))
                offset = r.get_width()
                self.screen.blit(r, (336, 218))
            else:
                offset = 0
            if cf == 1:
                self.screen.fill((0, 0, 255), Rect(336 + offset, 220, 2, 40))
                            
            pygame.display.update()
        self.full_draw_flag = 1

    def draw(self):
        if self.full_draw_flag:
            self.view.fill(0)
        self.t.draw_map(self.view, self.view_pos, self.full_draw_flag)
        self.full_draw_flag = 0
    
        self.draw_cursor()
        self.draw_status_bar()

        pygame.display.flip()

    def draw_cursor(self):
        c_x = self.cursor_pos[0]*self.t.tile_size - self.view_pos[0]
        c_y = self.cursor_pos[1]*self.t.tile_size - self.view_pos[1]
        cursor_rect = Rect(c_x, c_y, self.t.tile_size, self.t.tile_size)

        if self.old_cursor_pos and self.old_cursor_pos != self.cursor_pos:
            co_x = self.old_cursor_pos[0]*self.t.tile_size - self.view_pos[0]
            co_y = self.old_cursor_pos[1]*self.t.tile_size - self.view_pos[1]
            self.screen.fill(0, Rect(co_x, co_y, self.t.tile_size, self.t.tile_size))
            
            self.t.draw_region(self.view, pygame.Rect(self.old_cursor_pos[0], self.old_cursor_pos[1], 1, 1), self.view_pos, 1)
        
        pygame.draw.rect(self.view, self.cursor_colors[self.frame_count / 4 % 2], cursor_rect, 1)

    def draw_status_bar(self):
        if self.cur_tile != -1:
            self.t.draw_tile(self.screen, self.cur_tile, (24, 423))
        else:
            self.screen.fill(0, Rect(24, 423, self.t.tile_size, self.t.tile_size))

    def handle_events(self):
        e = pygame.event.poll()
        while e.type != NOEVENT:
            if e.type == QUIT:
                self.running = 0
            elif e.type == KEYDOWN:
                self.handle_keydown_event(e.key)
            e = pygame.event.poll()

    def handle_keydown_event(self, k):
        if k == K_ESCAPE:
            self.running = 0
        elif k == K_PERIOD: # Next tile
            self.cur_tile += 1
            if(self.cur_tile >= len(self.t.tileset)):
                self.cur_tile = -1
        elif k == K_COMMA:  # Previous tile
            self.cur_tile -= 1
            if(self.cur_tile < -1):
                self.cur_tile = len(self.t.tileset)-1
        elif k == K_RETURN: # "Grab" tile under cursor (like eyedropper)
            self.cur_tile = self.t.map[self.cursor_pos[1]][self.cursor_pos[0]]
        elif k == K_s:      # Save
            f = file("data/test.map", "w")
            pickle.dump(self.t.map, f)
            f.close()
        elif k == K_l:      # Load
            f = file("data/test.map", "r")
            self.t.load_map(pickle.load(f))
            f.close()
        elif k == K_n:      # New Map
            self.new_map()

    def handle_keyboard_state(self):
        self.scroll_view()

    def handle_mouse_state(self):
        if pygame.mouse.get_pressed()[0] and pygame.mouse.get_pos()[1] < 400:
            self.t.map[self.cursor_pos[1]][self.cursor_pos[0]] = self.cur_tile
            self.t.draw_region(self.view, pygame.Rect(self.cursor_pos[0], self.cursor_pos[1], 1, 1), self.view_pos, 1)

    def scroll_view(self):
        p = pygame.key.get_pressed()
        
        if p[K_UP]:
            self.view_pos = (self.view_pos[0], self.view_pos[1] - 10)
            self.full_draw_flag = 1
        if p[K_DOWN]:
            self.view_pos = (self.view_pos[0], self.view_pos[1] + 10)
            self.full_draw_flag = 1
        if p[K_LEFT]:
            self.view_pos = (self.view_pos[0] - 10, self.view_pos[1])
            self.full_draw_flag = 1
        if p[K_RIGHT]:
            self.view_pos = (self.view_pos[0] + 10, self.view_pos[1])
            self.full_draw_flag = 1

    def main(self):
        self.running = 1
        while self.running:
            self.frame_count += 1

            self.old_cursor_pos = self.cursor_pos
            mouse_pos = pygame.mouse.get_pos()
            self.cursor_pos = ((mouse_pos[0] + self.view_pos[0]) / self.t.tile_size, (mouse_pos[1] + self.view_pos[1]) / self.t.tile_size)
            self.cursor_pos = (min((max((self.cursor_pos[0], 0)), len(self.t.map[0])-1)), min((max((self.cursor_pos[1], 0)), len(self.t.map)-1)))

            self.draw()
            self.handle_keyboard_state()
            self.handle_mouse_state()
            self.handle_events()
            self.t.update()
            
        pygame.quit()

# Execute main function
if __name__ == "__main__":
    editor().main()
