"""
Sprite Repository
"""

import pygame, Numeric, math, StringIO
from pygame.locals import *

class sprite_repository:
    def __init__(self, filename):
        self.load_repository(filename)

    def load_repository(self, filename):
        self.sprites = dict()
        
        import xml.dom.minidom, zipfile
        d = xml.dom.minidom.parse(filename + ".xml")
        z = zipfile.ZipFile(filename + ".zip")
        for s in d.documentElement.getElementsByTagName("sprite"):
            cur_sprite = sr_sprite(s.getAttribute("name"), s.getAttribute("width"), s.getAttribute("height"))
            cur_sprite.load_animations(s, z)
            self.sprites[cur_sprite.name] = cur_sprite
        z.close()

class sr_sprite:
    def __init__(self, name, width, height):
        self.name = name
        self.width, self.height = int(width), int(height)
        self.animations = dict()

    def load_animations(self, s, z):
        for a in s.getElementsByTagName("animation"):
            cur_anim = sr_animation(a.getAttribute("name"), a.getAttribute("type"))
            cur_anim.load_frames(a, self.name, z, (self.width, self.height))
            self.animations[cur_anim.name] = cur_anim

class sr_animation:
    def __init__(self, name, type):
        self.name = name
        self.type = type
        self.frames = []

    def load_frames(self, a, sn, z, size):
        for f in a.getElementsByTagName("frame"):
            if a.getAttribute("type") == "None":
                time = 0
            else:
                time = int(f.getAttribute("time"))
                
            img = pygame.image.load(StringIO.StringIO(z.read(sn + "/" + f.getAttribute("filename"))), f.getAttribute("filename"))
            offset = (int(f.getAttribute("offset_x")), int(f.getAttribute("offset_y")))

            collision_rects = []            
            for cr in f.getElementsByTagName("collision_rect"):
                r = Rect(int(cr.getAttribute("x")), int(cr.getAttribute("y")), int(cr.getAttribute("w")), int(cr.getAttribute("h")))
                t = cr.getAttribute("type")
                collision_rects.append(sr_collision_rect(r, t))
            if len(collision_rects) == 0:
                collision_rects.append(sr_collision_rect(Rect(offset[0], offset[1], offset[0] + img.get_width(), offset[1] + img.get_height()), "Vulnerable"))
            
            self.frames.append(sr_frame(img, time, offset, collision_rects, size))

class sr_frame:
    def __init__(self, img, time, offset, collision_rects, size):
        self.img = img.convert_alpha()
        self.flip_img = pygame.transform.flip(img.convert_alpha(), 1, 0)
        self.time = time
        self.offset = offset
        
        self.collision_rects = collision_rects
        self.flip_collision_rects = []
        for cr in collision_rects:
            self.flip_collision_rects.append(sr_collision_rect(Rect(size[0] - cr.rect.x - cr.rect.w, cr.rect.y, cr.rect.w, cr.rect.h), cr.type))

        # Calculate bounds!  Eew!  I know!
        self.cr_bounds = Rect(self.collision_rects[0].rect.x, self.collision_rects[0].rect.y, self.collision_rects[0].rect.w, self.collision_rects[0].rect.h)
        self.cr_flip_bounds = Rect(self.flip_collision_rects[0].rect.x, self.flip_collision_rects[0].rect.y, self.flip_collision_rects[0].rect.w, self.flip_collision_rects[0].rect.h)
        self.cr_bounds.x = min([r.rect.x for r in self.collision_rects])
        self.cr_bounds.y = min([r.rect.y for r in self.collision_rects])
        self.cr_bounds.w = max([r.rect.x+r.rect.w-self.cr_bounds.x for r in self.collision_rects])
        self.cr_bounds.h = max([r.rect.y+r.rect.h-self.cr_bounds.y for r in self.collision_rects])
        self.cr_flip_bounds.x = min([r.rect.x for r in self.flip_collision_rects])
        self.cr_flip_bounds.y = min([r.rect.y for r in self.flip_collision_rects])
        self.cr_flip_bounds.w = max([r.rect.x+r.rect.w-self.cr_flip_bounds.x for r in self.flip_collision_rects])
        self.cr_flip_bounds.h = max([r.rect.y+r.rect.h-self.cr_flip_bounds.y for r in self.flip_collision_rects])

class sr_collision_rect:
    def __init__(self, r, t):
        self.rect = r
        self.type = t
