"""
The Player
"""

import pygame, math
from pygame.locals import *
from my_sprite import my_sprite

class player(my_sprite):
    def __init__(self, game, loc, initial_facing, initial_state):
        my_sprite.__init__(self, game, loc, "Player", initial_facing, initial_state, False, True)
        
        self.jump_flag, self.attack_flag = 0, 0
        self.invuln_time = -1 # Player will be invulnerable until this frame

    def draw(self, surf):
        # Flash if invulnerable
        if self.invuln_time < self.game.frame_count or (self.game.frame_count // 8) % 2 == 0:
            my_sprite.draw(self, surf)
        
    def update_state(self):
        """Update the player's state."""
        # Decceleration
        if self.what_under:
            if self.state in ("Standing Slash", "Slide Kick", "Standing Suffer"):
                d = 0.05
            else:
                d = 0.5
        else:
            d = 0.05

        if self.vel[0] > 0:
            self.vel[0] = max(self.vel[0] - d, 0)
        elif self.vel[0] < 0:
            self.vel[0] = min(self.vel[0] + d, 0)
        
        # Player states can be affected by keyboard or state machine
        self.handle_keyboard_state()
        self.handle_state_machine()
        
    def handle_state_machine(self):
        """Handle the internal state changes (i.e. those not directly dependant on keyboard input)."""
        # Getting hit
        if (self.invuln_time < self.game.frame_count) and (self.cr_flags['Vulnerable']):
            self.invuln_time = self.game.frame_count + 300
            self.set_state("Standing Suffer")
            if self.facing == 0:
                self.vel[0] = -1.5
            else:
                self.vel[0] = 1.5
        
        # Landing after having jumped
        if self.what_under:
            if self.state in ("Jump (Down)", "Jump (After Slash)", "Jumping Slash"):
                if self.vel[0] == 0:
                    if pygame.key.get_pressed()[K_DOWN] or pygame.key.get_pressed()[K_KP2]:
                        self.set_state("Crouch")
                    else:
                        self.set_state("Stand")
                else:
                    self.set_state("Run")

        # Falling down (> 0.5 helps with landing bug, but causes bug with jumping up onto platforms :S)
        if self.vel[1] > 0 and not (self.state in ("Jumping Slash", "Standing Suffer")):
            self.set_state("Jump (Down)")

        # Returning to normal after temporary state change
        if self.anim_runs > 0:
            if self.state == "Standing Slash":
                self.set_state("Stand")
            if self.state == "Jumping Slash":
                self.set_state("Jump (After Slash)")
            if self.state == "Slide Kick":
                self.set_state("Crouch")
            if self.state == "Standing Suffer":
                self.set_state("Stand")
        
    def handle_keyboard_state(self):
        # Attack
        if (pygame.key.get_pressed()[K_LCTRL] or pygame.key.get_pressed()[K_RCTRL]):
            if self.attack_flag == 0:
                # Standing Slash
                if self.state in ("Stand", "Run"):
                    self.set_state("Standing Slash")
                # Jumping Slash
#                elif self.state in ("Jump (Up)", "Jump (Down)"):
#                    self.set_state("Jumping Slash")
                # Slide kick
                elif self.state == "Crouch":
                    if (pygame.key.get_pressed()[K_LCTRL] or pygame.key.get_pressed()[K_RCTRL]):
                        self.set_state("Slide Kick")
                        if self.facing == 0:
                            self.vel[0] = 3
                        else:
                            self.vel[0] = -3
            self.attack_flag = 1
        else:
            self.attack_flag = 0

        # Jump
        if pygame.key.get_pressed()[K_SPACE]:
            if self.jump_flag == 0:
                if self.state in ("Stand", "Run") or (self.state == "Slide Kick" and self.cur_frame >= 3):
                    self.what_under = 0
                    self.set_state("Jump (Up)")
                    self.vel[1] = -3.5
                elif self.state == "Crouch":
                    if self.what_under == 2:
                        self.r.y = self.r.y + 1
            else:
                if self.state == "Jump (Up)" and (pygame.time.get_ticks() - self.last_time >= 75) and (pygame.time.get_ticks() - self.last_time <= 125):
                    self.vel[1] = -3
            self.jump_flag = 1
        else:
            self.jump_flag = 0

        # Levitation                    
        if pygame.key.get_pressed()[K_UP] or pygame.key.get_pressed()[K_KP8]:
            pass
#            self.vel[1] = max(self.vel[1] - 0.25, -5)

        # Crouch
        if pygame.key.get_pressed()[K_DOWN] or pygame.key.get_pressed()[K_KP2]:
            if self.state == "Stand":
                self.set_state("Crouch")
        else:
            if self.state == "Crouch":
                self.set_state("Stand")

        # Running or Falling (left or right)
        if pygame.key.get_pressed()[K_LEFT] or pygame.key.get_pressed()[K_KP4]:
            if self.state == "Stand" or (self.state == "Run" and self.vel[0] > 0):
                self.set_state("Run")
            if not self.state in ("Standing Slash", "Standing Suffer", "Slide Kick"):
                self.facing = 1
            if self.state in ("Stand", "Run", "Jump (Up)", "Jump (Down)",  "Jumping Slash", "Jump (After Slash)"):
                self.vel[0] = -1.75
        elif pygame.key.get_pressed()[K_RIGHT] or pygame.key.get_pressed()[K_KP6]:
            if self.state == "Stand" or (self.state == "Run" and self.vel[0] < 0):
                self.set_state("Run")
            if not self.state in ("Standing Slash", "Standing Suffer", "Slide Kick"):
                self.facing = 0
            if self.state in ("Stand", "Run", "Jump (Up)", "Jump (Down)",  "Jumping Slash", "Jump (After Slash)"):
                self.vel[0] = 1.75
        else:
            if self.state == "Run":
                self.set_state("Stand") # Maybe a transition state from Run to Stand here instead?
